﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace LightSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		LightSensor openCh = null;
		LightSensor lux = null;
		private ErrorEventBox errorBox;
		private DataGraphBox luxGraphBox;
		formCleaner cleaner;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(LightSensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			this.Size = this.MinimumSize;

			errorBox = new ErrorEventBox(this);
			luxGraphBox = new DataGraphBox(this, "Lux");
			lux = openCh ?? openArgs.makeChannel<LightSensor>();

			this.Size = this.MinimumSize;

			lux.Attach += lux_attach;
			lux.Detach += lux_detach;
			lux.Error += lux_error;
			lux.IlluminanceChange += lux_change;
			lux.PropertyChange += lux_propertychange;

			try {
				lux.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}
		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			lux.Attach -= lux_attach;
			lux.Detach -= lux_detach;
			lux.Error -= lux_error;
			lux.IlluminanceChange -= lux_change;
			lux.PropertyChange -= lux_propertychange;
			lux.Close();
		}

		void lux_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			LightSensor attachedDevice = (LightSensor)sender;

			luxGraphBox.SetTitle(DataGraphBox.GenerateTitle("Illuminance", (Phidget)attachedDevice));

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "lx";
				changeTriggerTrk.minTxt = attachedDevice.MinIlluminanceChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinIlluminanceChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxIlluminanceChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxIlluminanceChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.IlluminanceChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.IlluminanceChangeTrigger;
			} catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

			settingsBox.Visible = true;
			outputBox.Visible = true;

		}
		void lux_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}
		void lux_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				luxGraphBox.addGap();
		}

		private void lux_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
					{
						dataIntervalTrk.Value = lux.DataInterval;
						updateGraphDataInterval();
					}
					break;
				case "IlluminanceChangeTrigger":
					changeTriggerTrk.Value = lux.IlluminanceChangeTrigger;
					break;
				case "DataRate":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
					{
						dataIntervalTrk.Value = lux.DataRate;
						updateGraphDataInterval();
					}
					break;
			}
		}
		void lux_change(object sender, Phidget22.Events.LightSensorIlluminanceChangeEventArgs e) {
			try {
				luxvalue.Text = e.Illuminance.ToString() + " lx";
				luxGraphBox.addData(e.Illuminance, lux.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage("Error reading illuminance: " + ex.Message); }
		}

        private void updateGraphDataInterval()
		{
			try
			{
				if (lux.IlluminanceChangeTrigger == 0)
					luxGraphBox.DataInterval = (1000.0 / lux.DataRate);
				else
					luxGraphBox.DataInterval = null;
			}
			catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

        private void luxGraphButton_Click(object sender, EventArgs e) {
			if (!luxGraphBox.Visible)
				luxGraphBox.Show();
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e)
		{
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e)
		{
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
				{
					try
					{
						double temp = dataIntervalTrk.Value;
						lux.DataInterval = (int)temp;
						if (lux.DataInterval < (int)temp)
						{
							while (lux.DataInterval != (int)temp)
							{
								temp++;
								lux.DataInterval = (int)temp;
							}
						}
						else
						{
							while (lux.DataInterval != (int)temp)
							{
								temp--;
								lux.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = lux.DataInterval;
						dataIntervalTrk.Value = lux.DataInterval;



						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
				else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
				{
					try
					{
						lux.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
			else //changetrigger control
			{
				try
				{
					lux.IlluminanceChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
		{
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = lux.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = lux.DataRate;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					if (lux.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = lux.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = lux.DataInterval;
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}
